#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright © 2020 Aleksandr Suvorov
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
# -----------------------------------------------------------------------------
# Aleksandr Suvorov (CyberLegioner)
# Github: https://github.com/CyberLegioner/smartcleaner
# Donate: https://money.yandex.ru/to/4100110928527458
# Site: https://py2.ru
# Email: smart-py@yandex.ru
# PyPi: https://pypi.org/project/smartcleaner/
# -----------------------------------------------------------------------------

import os
import sys
import webbrowser
import locale
from datetime import datetime
from PySide2.QtCore import QThread, Signal, Qt
from PySide2.QtWidgets import (QApplication, QFileDialog, QMessageBox, QLabel, QHBoxLayout, QVBoxLayout, QPushButton,
                               QSpinBox, QCheckBox, QTextBrowser, QLCDNumber, QListWidget, QWidget, QAbstractItemView)
from scleaner.smart import SmartData
from scleaner.cleaner import Cleaner


class MyLang:
    def __init__(self):
        self.TXT_OK = self.lang(['Выполнено!', 'Completed!'])
        self.TXT_ZERO_FILE = self.lang(['Обнуляем файл: ', 'Zeroing the file: '])
        self.TXT_ZERO_FILE_ERR = self.lang(['Ошибка при обнулении файла!', 'Error updating the file!'])
        self.TXT_SHRED_FILE = self.lang(['Затираем файл: ', 'Mashing the file: '])
        self.TXT_ERR_SHRED_FILE = self.lang(['Ошибка при затирании файла!', 'Error when deleting the file!'])
        self.TXT_DEL_FILE = self.lang(['Удаляем файл: ', 'Delete file:'])
        self.TXT_ERR_DEL_FILE = self.lang(['Ошибка при удалении файла', 'Error deleting a file'])
        self.TXT_DEL_DIR = self.lang(['Удаляем папку: ', 'Deleting a folder: '])
        self.TXT_ERR_DEL_DIR = self.lang(['Ошибка при удалении папки!', 'Error when deleting a folder!'])
        self.TXT_FILE_NOT_EXIST = self.lang(['Ошибка! Файл не существует!', 'Error! The file does not exist!'])
        self.TXT_DIR_NOT_EXISTS = self.lang(['Ошибка! Папка не существует!', 'Error! The folder does not exist!'])
        self.TXT_ZERO_END = self.lang(['Обнуление завершено. Обнулено файлов: ', 'Reset completed. Reset files : '])
        self.TXT_ERRS = self.lang(['Ошибок: ', 'Errors: '])
        self.TXT_END_DEL = self.lang(['Удаление завершено. Удалено файлов: ', 'The deletion is complete. Deleted '
                                                                              'files : '])
        self.TXT_DEL_DIRS = self.lang(['Удалено папок: ', 'Deleted folders: '])
        self.TXT_ERRORS = self.lang(['Ошибок: ', 'Errors: '])
        self.TXT_SMART_REPORT = self.lang(['Smart Cleaner отчет о работе: ', 'Smart Cleaner report on the work: '])
        self.TXT_PLATFORMS = self.lang(['Платформа: ', 'Platform: '])
        self.TXT_SHREDS = self.lang(['Проходов: ', 'Passageways: '])
        self.TXT_ZERO_COUNT = self.lang(['Обнулено файлов: ', 'Reset files : '])
        self.TXT_SHRED_FILES = self.lang(['Затерто файлов: ', 'Destroyed files : '])
        self.TXT_DEL_FILES = self.lang(['Удалено файлов: ', 'Deleted files : '])
        self.TXT_ERRS_SHOW = self.lang(['Ошибки: ', 'Errors: '])

    @staticmethod
    def lang(my_list):
        if locale.getdefaultlocale()[0] == 'ru_RU':
            return my_list[0]
        else:
            return my_list[1]


class SmartCleaner(QThread):
    signal = Signal(str)

    def __init__(self):
        super().__init__()
        self.decor = '-' * 115
        self.__errors = []
        self.shreds = 30
        self.status_del_dir = False
        self.cleaner = Cleaner()
        self.my_data = SmartData()
        self.my_msg = MyLang()
        self.flag = None

    def run(self):
        self.cleaner.reset_count()
        self.__clear_errors()
        self._from_start()
        if not self.flag == 'zero' and self.status_del_dir:
            self._del_dirs()
        self._from_finish()

    def _from_finish(self):
        if self.__errors:
            self._emit_msg(self.decor)
            self._emit_msg(f'{self.my_msg.TXT_ERRS_SHOW}')
            for err in self.__errors:
                self._emit_msg(err)
        self._emit_msg(self.decor)
        if self.flag == 'zero':
            self._emit_msg(f'{self.my_msg.TXT_ZERO_END}{self.cleaner.count_zero_files} '
                           f'| {self.my_msg.TXT_ERRS} {len(self.__errors)}')
        else:
            self._emit_msg(f'{self.my_msg.TXT_END_DEL}{self.cleaner.count_del_files} '
                           f'| {self.my_msg.TXT_DEL_DIRS}{self.cleaner.count_del_dirs} '
                           f'| {self.my_msg.TXT_ERRORS}{len(self.__errors)}')
            self.my_data.clear_data()

    def _from_start(self):
        self._emit_msg(f'{self.my_msg.TXT_SMART_REPORT}{datetime.now()}\n{self.decor}\n'
                       f' {self.my_msg.TXT_PLATFORMS}{"Linux" if os.name == "posix" else "Windows"} '
                       f"| {self.my_msg.TXT_SHREDS}{self.shreds} ")
        for file in self.my_data.get_files_gen():
            self._emit_msg(self.decor)
            if os.path.exists(file):
                status, msg, err_msg = self._start_method(file)
                self._emit_msg(msg)
                if status:
                    self._emit_msg(self.my_msg.TXT_OK)
                else:
                    self._emit_msg(err_msg)
                    self.__errors.append(file)
                self._emit_msg(self._get_counts)
            else:
                self._emit_msg(self.my_msg.TXT_FILE_NOT_EXIST)

    def _start_method(self, file):
        if self.flag == 'zero':
            status = self.cleaner.zero_file(file)
            msg = f'{self.my_msg.TXT_ZERO_FILE} {file}'
            err_msg = self.my_msg.TXT_ZERO_FILE_ERR
        elif self.flag == 'shred':
            status = self.cleaner.shred_file(file, self.shreds)
            msg = f'{self.my_msg.TXT_SHRED_FILE} {file}'
            err_msg = self.my_msg.TXT_ERR_SHRED_FILE
        else:
            status = self.cleaner.del_file(file)
            msg = f'{self.my_msg.TXT_DEL_FILE} {file}'
            err_msg = self.my_msg.TXT_ERR_DEL_FILE
        return status, msg, err_msg

    @property
    def _get_counts(self):
        if self.flag == 'zero':
            return f'{self.my_msg.TXT_ZERO_COUNT}{self.cleaner.count_zero_files} | {self.my_msg.TXT_ERRORS}{self.num_errors}'
        elif self.flag == 'shred':
            return f'{self.my_msg.TXT_SHRED_FILES}{self.cleaner.count_del_files} | {self.my_msg.TXT_ERRORS}{self.num_errors}'
        else:
            return f'{self.my_msg.TXT_DEL_FILES}{self.cleaner.count_del_files} | {self.my_msg.TXT_ERRORS}{self.num_errors}'

    def _del_dirs(self):
        for path in self.my_data.get_folders_gen():
            self._emit_msg(self.decor)
            self._emit_msg(f'{self.my_msg.TXT_DEL_DIR} {path}')
            if os.path.exists(path):
                if self.cleaner.del_dir(path):
                    self._emit_msg(self.my_msg.TXT_OK)
                else:
                    self._emit_msg(self.my_msg.TXT_ERR_DEL_DIR)
                    self.__errors.append(path)
            else:
                self._emit_msg(self.my_msg.TXT_DIR_NOT_EXISTS)
            self._emit_msg(f'{self.my_msg.TXT_DEL_DIRS}{self.cleaner.count_del_dirs} '
                           f'| {self.my_msg.TXT_ERRORS}{self.num_errors}')

    def _emit_msg(self, msg):
        self.signal.emit(msg)

    def get_errors(self) -> list:
        return self.__errors

    def __clear_errors(self) -> None:
        self.__errors.clear()

    @property
    def num_errors(self) -> int:
        return len(self.__errors)


class WinTxt:
    def __init__(self):
        self.TXT_NAME = self.lang(('Smart Cleaner - программа для уничтожения данных | http://www.py2.ru/smart-cleaner',
                                   'Smart Cleaner - data wiping program | http://www.py2.ru/smart-cleaner'))
        self.TXT_LABEL_LOGO = self.lang((f'Smart Cleaner<sup> 2.1</sup>', 'Smart Cleaner<sup> 2.1</sup>'))
        self.TXT_LABEL_FILES = self.lang(('Файлы', 'Files'))
        self.TXT_LABEL_DIRS = self.lang(('Папки', 'Folders'))
        self.TXT_LABEL_ERRORS = self.lang(('Ошибки', 'Errors'))
        self.TXT_LABEL_CONS = self.lang(('Информационная консоль:', 'Information console:'))
        self.TXT_BTN_HELP = self.lang(('Помощь', 'Help'))
        self.TXT_BTN_CONSOLE_CLEAR = self.lang(('Сброс', 'Reset'))
        self.TXT_BTN_SAVE_LOG = self.lang(('Сохранить отчет', 'Save report'))
        self.TXT_BTN_DONATE = self.lang(('Donate | 4276 4417 5763 7686', 'Donate | 4276 4417 5763 7686'))
        self.TXT_URL_OPEN = self.lang(("Сайт", 'Website'))
        self.TXT_DEL_DIRS_CHB = self.lang(("Удалять папки", "Delete folders"))
        self.TXT_LABEL_SHRED = self.lang(('Проходов:', 'Rewrite:'))
        self.TXT_FOLDER = self.lang(('+ Папка', '+ Folder'))
        self.TXT_FILES = self.lang(('+ Файлы', '+ Files'))
        self.TXT_REMOVE_ITEM = self.lang(('- Убрать', '- Remove'))
        self.TXT_ZEROING = self.lang(('Обнулить', 'Zeroing'))
        self.TXT_ERASING = self.lang(('Затереть', 'Erasing'))
        self.TXT_DELETE = self.lang(('Удалить', 'Delete'))
        self.TXT_URL_DONATE = self.lang(('https://money.yandex.ru/to/4100110928527458',
                                         'https://money.yandex.ru/to/4100110928527458'))
        self.TXT_SITE = self.lang(('http://www.py2.ru/smart-cleaner', 'http://www.py2.ru/smart-cleaner'))
        self.TXT_WELCOME = self.lang(
            (f'Smart Cleaner v2.0 | {datetime.now()}\nДобро пожаловать в программу для затирания, '
             f'обнуления и удаления данных.',
             f'Smart Cleaner v2.0 | {datetime.now()}\nWelcome to the program for mashing, '
             f'zeroing and deleting data.'))
        self.TXT_DON_TOOL_TIP = self.lang(('Мы будем благодарны вам за любую финансовую поддержку.\n'
                                           'Это поможет программе развиваться и оставаться бесплатной.\nСпасибо!',
                                           'We will be grateful for any financial support.\nThis will help the program '
                                           'develop and remain free.\nThanks!'))
        self.TXT_ERR_NO_DATA = self.lang(('Нет данных для затирания!!!', 'There is no data for mashing!!!'))
        self.TXT_WAR_DATA_SHRED = self.lang(
            ('Данные будут уничтожены, вы уверены?', 'The data will be destroyed, are you sure?')
        )
        self.TXT_WARNING = self.lang(('Внимание!', 'Warning!'))
        self.TXT_ZERO_START = self.lang(('Запущено обнуление файлов.', 'Files are reset to zero.'))
        self.TXT_SHRED_START = self.lang(('Запущено затирание файлов.', 'File mashing is started.'))
        self.TXT_DEL_START = self.lang(('Запущено удаление файлов.', 'File deletion started.'))
        self.TXT_ADD_PATH = self.lang(('Путь успешно добавлен!', 'The path was added successfully!'))
        self.TXT_ERR_ADD_PATH = self.lang(('Ошибка при добавлении или путь был добавлен ранее!!!',
                                           'Error when adding or the path was added earlier!!!'))
        self.TXT_REMOVE_PATH = self.lang(('Путь успешно удален!!!', 'The path was successfully deleted!!!'))
        self.TXT_TO_EXIT = self.lang(('Вы уверены что хотите завершить работу программы?',
                                      'Are you sure you want to terminate the program?'))
        self.TXT_ZERO = self.lang(('Обнуление', 'Reset'))
        self.TXT_FILES_ZERO = self.lang(('Обнулено файлов: ', 'Reset files: '))
        self.TXT_TO_SHRED = self.lang(('Затирание', 'Mashing'))
        self.TXT_SHREDS = self.lang(('Затерто файлов: ', 'Destroyed files: '))
        self.TXT_TO_DEL = self.lang(('Удаление', 'Delete'))
        self.msg_from_delete = self.lang(('Удалено файлов: ', 'Deleted files: '))
        self.msg_ok_end = self.lang(('успешно завершено!', 'completed successfully!!!'))
        self.TXT_DIRS_DEL = self.lang(('Удалено папок: ', 'Deleted folders: '))
        self.TXT_ERRORS = self.lang(('Ошибок: ', 'Errors: '))
        self.TXT_SAVE_LOG_ERR = self.lang(('При сохранении файла произошла ошибка, отчет не был сохранен!',
                                           'An error occurred when saving the file and the report was not saved!'))
        self.TXT_OK_SAVE_LOG = self.lang(('Отчет был успешно сохранен!  Файл: ',
                                          'The report was saved successfully! File: '))
        self.TXT_ERR = self.lang(('Ошибка!', 'Error!'))
        self.TXT_NOT_DATA_TO_SAVE = self.lang(('Нет данных для сохранения!', 'There is no data to save!'))
        self.TXT_SELECT_DIR = self.lang(('Выберите папку для добавления: ', 'Select the folder to add: '))
        self.TXT_SELECT_FILES = self.lang(('Выберите файлы для добавления: ', 'Select files to add: '))
        self.TXT_EXIT = self.lang(('Выход', 'Exit'))
        self.TXT_LABEL_DONATE = self.lang(('Donate: Сбербанк - 4276 4417 5763 7686 '
                                           '| PayPal: paypal.me/py2ru | Яндекс деньги: smart-py@yandex.ru',
                                           'Donate: SberBank - 4276 4417 5763 7686 '
                                           '| PayPal: paypal.me/py2ru | Yandex Money: smart-py@yandex.ru'))

    @staticmethod
    def lang(my_list):
        if locale.getdefaultlocale()[0] == 'ru_RU':
            return my_list[0]
        else:
            return my_list[1]


class MyWindow(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.log_file_name = 'log_smart_cleaner.txt'
        self.set_text = WinTxt()
        self.setWindowTitle(self.set_text.TXT_NAME)
        self.my_logs = []
        self.method = 'shred'
        self.decor = '-' * 115
        self.label_donate = QLabel(self.set_text.TXT_LABEL_DONATE)
        self.label_donate.setAlignment(Qt.AlignCenter)
        self.label_logo = QLabel(self.set_text.TXT_LABEL_LOGO)
        self.label_logo.setAlignment(Qt.AlignCenter)
        self.label_logo.setStyleSheet('font-size: 48px;')
        self.label_files = QLabel(self.set_text.TXT_LABEL_FILES)
        self.label_files.setStyleSheet("color: rgb(84, 180, 40);")
        self.label_dirs = QLabel(self.set_text.TXT_LABEL_DIRS)
        self.label_dirs.setStyleSheet("color: rgb(177, 98, 42);")
        self.label_errors = QLabel(self.set_text.TXT_LABEL_ERRORS)
        self.label_errors.setStyleSheet("color: rgb(255, 68, 44);")
        self.lcd_files = QLCDNumber()
        self.lcd_files.setSegmentStyle(QLCDNumber.Flat)
        self.lcd_files.setStyleSheet("color: rgb(84, 180, 40);")
        self.lcd_dirs = QLCDNumber()
        self.lcd_dirs.setSegmentStyle(QLCDNumber.Flat)
        self.lcd_dirs.setStyleSheet("color: rgb(177, 98, 42);")
        self.lcd_errors = QLCDNumber()
        self.lcd_errors.setSegmentStyle(QLCDNumber.Flat)
        self.lcd_errors.setMinimumHeight(30)
        self.lcd_errors.setStyleSheet("color: rgb(255, 68, 44);")
        self.lcd_files.setDigitCount(15)
        self.lcd_dirs.setDigitCount(15)
        self.lcd_errors.setDigitCount(15)
        self.h_box1 = QHBoxLayout()
        self.h_box1.addWidget(self.label_dirs)
        self.h_box1.addWidget(self.label_files)
        self.h_box1.addWidget(self.label_errors)
        self.h_box2 = QHBoxLayout()
        self.h_box2.addWidget(self.lcd_dirs)
        self.h_box2.addWidget(self.lcd_files)
        self.h_box2.addWidget(self.lcd_errors)
        self.label_cons = QLabel(self.set_text.TXT_LABEL_CONS)
        self.text_browser = QTextBrowser()
        self.text_browser.setText(self.set_text.TXT_WELCOME)
        self.btn_console_clear = QPushButton(self.set_text.TXT_BTN_CONSOLE_CLEAR)
        self.btn_save_log = QPushButton(self.set_text.TXT_BTN_SAVE_LOG)
        self.btn_save_log.setDisabled(True)
        self.btn_donate = QPushButton(self.set_text.TXT_BTN_DONATE)
        self.btn_donate.setToolTip(self.set_text.TXT_DON_TOOL_TIP)
        self.btn_open_url = QPushButton(self.set_text.TXT_URL_OPEN)
        self.h_box3 = QHBoxLayout()
        self.h_box3.addWidget(self.btn_open_url)
        self.h_box3.addWidget(self.btn_donate)
        self.h_box3.addStretch(1)
        self.h_box3.addWidget(self.btn_save_log)
        self.h_box3.addWidget(self.btn_console_clear)
        self.chb_del_dirs = QCheckBox(self.set_text.TXT_DEL_DIRS_CHB)
        self.chb_del_dirs.setChecked(True)
        self.label_shred = QLabel(self.set_text.TXT_LABEL_SHRED)
        self.spin_box = QSpinBox()
        self.spin_box.setMinimum(1)
        self.spin_box.setMaximum(1000)
        self.spin_box.setValue(30)
        self.h_box4 = QHBoxLayout()
        self.h_box4.addWidget(self.chb_del_dirs)
        self.h_box4.addWidget(self.label_shred)
        self.h_box4.addWidget(self.spin_box)
        self.h_box4.addStretch(1)
        self.list_widget = QListWidget()
        self.list_widget.setSelectionMode(QAbstractItemView.ExtendedSelection)
        self.btn_add_folder = QPushButton(self.set_text.TXT_FOLDER)
        self.btn_add_files = QPushButton(self.set_text.TXT_FILES)
        self.btn_remove_item = QPushButton(self.set_text.TXT_REMOVE_ITEM)
        self.btn_zero_files = QPushButton(self.set_text.TXT_ZEROING)
        self.btn_shred_files = QPushButton(self.set_text.TXT_ERASING)
        self.btn_del_files = QPushButton(self.set_text.TXT_DELETE)
        self.h_box5 = QHBoxLayout()
        self.h_box5.addWidget(self.btn_add_folder)
        self.h_box5.addWidget(self.btn_add_files)
        self.h_box5.addWidget(self.btn_remove_item)
        self.h_box5.addStretch(1)
        self.h_box5.addWidget(self.btn_zero_files)
        self.h_box5.addWidget(self.btn_shred_files)
        self.h_box5.addWidget(self.btn_del_files)
        self.v_box = QVBoxLayout()
        self.v_box.addWidget(self.label_logo)
        self.v_box.addLayout(self.h_box1)
        self.v_box.addLayout(self.h_box2)
        self.v_box.addWidget(self.label_cons)
        self.v_box.addWidget(self.text_browser)
        self.v_box.addLayout(self.h_box3)
        self.v_box.addLayout(self.h_box4)
        self.v_box.addWidget(self.list_widget)
        self.v_box.addLayout(self.h_box5)
        self.v_box.addWidget(self.label_donate)
        self.setLayout(self.v_box)
        self.my_cleaner = SmartCleaner()
        self.btn_donate.clicked.connect(lambda: webbrowser.open(self.set_text.TXT_URL_DONATE))
        self.btn_open_url.clicked.connect(lambda: webbrowser.open(self.set_text.TXT_SITE))
        self.btn_console_clear.clicked.connect(self._clear_console)
        self.btn_add_folder.clicked.connect(self.__add_dir)
        self.btn_add_files.clicked.connect(self.__add_files)
        self.btn_remove_item.clicked.connect(self.__remove_items)
        self.btn_zero_files.clicked.connect(self.__from_start)
        self.btn_shred_files.clicked.connect(self.__from_start)
        self.btn_del_files.clicked.connect(self.__from_start)
        self.btn_save_log.clicked.connect(self.__save_log)
        self.my_cleaner.signal.connect(self.__update_information)
        self.my_cleaner.started.connect(self.__at_start)
        self.my_cleaner.finished.connect(self.__at_finish)

    def __from_start(self):
        if not self.my_cleaner.my_data.avail_of_data:
            self.__show_msg(self.set_text.TXT_WARNING, self.set_text.TXT_ERR_NO_DATA)
        else:
            reply = QMessageBox.question(self, self.set_text.TXT_WARNING, self.set_text.TXT_WAR_DATA_SHRED,
                                         QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
            if reply == QMessageBox.Yes:
                self.text_browser.append(self.decor)
                sender = self.sender()
                if sender.text() in ['Zeroing', 'Обнулить']:
                    self.method = 'zero'
                    self.my_cleaner.flag = self.method
                    self.text_browser.append(self.set_text.TXT_ZERO_START)
                elif sender.text() in ['Erasing', 'Затереть']:
                    self.method = 'shred'
                    self.my_cleaner.flag = self.method
                    self.text_browser.append(self.set_text.TXT_SHRED_START)
                else:
                    self.method = 'del'
                    self.my_cleaner.flag = self.method
                    self.text_browser.append(self.set_text.TXT_DEL_START)
                self.my_cleaner.shreds = self.spin_box.value()
                self.my_cleaner.status_del_dir = self.chb_del_dirs.isChecked()
                self.my_cleaner.start()

    def __at_start(self) -> None:
        self.__from_disable(True)

    def __at_finish(self) -> None:
        if self.method != 'zero':
            self.list_widget.clear()
        self.__update_lcd()
        self.__from_disable(False)
        self.my_logs.append(self.text_browser.toPlainText())
        self.btn_save_log.setDisabled(False)
        self.__finish_msg()

    def __finish_msg(self) -> None:
        if self.method == 'zero':
            msg = (self.set_text.TXT_ZERO, self.set_text.TXT_FILES_ZERO)
            count = self.my_cleaner.cleaner.count_zero_files
        elif self.method == 'shred':
            msg = (self.set_text.TXT_TO_SHRED, self.set_text.TXT_SHREDS)
            count = self.my_cleaner.cleaner.count_del_files
        else:
            msg = (self.set_text.TXT_TO_DEL, self.set_text.msg_from_delete)
            count = self.my_cleaner.cleaner.count_del_files
        self.__show_msg(self.set_text.TXT_WARNING,
                        f'{msg[0]} {self.set_text.msg_ok_end}\n'
                        f' {msg[1]} {count}\n '
                        f'{self.set_text.TXT_DIRS_DEL}{self.my_cleaner.cleaner.count_del_dirs}\n '
                        f'{self.set_text.TXT_ERRORS}{self.my_cleaner.num_errors}')

    def __save_log(self) -> None:
        if self.my_logs:
            os.chdir(os.path.dirname(os.path.abspath(__file__)))
            try:
                with open(self.log_file_name, 'a+', encoding="utf-8") as f:
                    for log in self.my_logs:
                        print(f'\n{log}', file=f)
                        f.flush()
            except OSError:
                self.__show_msg(self.set_text.TXT_WARNING, self.set_text.TXT_SAVE_LOG_ERR)
            else:
                self.__show_msg(self.set_text.TXT_WARNING, f'{self.set_text.TXT_OK_SAVE_LOG}{self.log_file_name}')
                self.btn_save_log.setDisabled(True)
                self.my_logs.clear()
        else:
            self.__show_msg(self.set_text.TXT_ERR, self.set_text.TXT_NOT_DATA_TO_SAVE)
        self._clear_console()
        self.btn_save_log.setDisabled(True)

    def __from_disable(self, status: bool) -> None:
        self.btn_zero_files.setDisabled(status)
        self.btn_remove_item.setDisabled(status)
        self.btn_add_folder.setDisabled(status)
        self.btn_shred_files.setDisabled(status)
        self.btn_console_clear.setDisabled(status)
        self.btn_save_log.setDisabled(status)
        self.btn_add_files.setDisabled(status)
        self.btn_del_files.setDisabled(status)
        self.chb_del_dirs.setDisabled(status)
        self.spin_box.setDisabled(status)
        self.list_widget.setDisabled(status)

    def __update_information(self, s: str) -> None:
        self.text_browser.append(s)
        self.__update_lcd()

    def __add_path(self, path: str) -> None:
        if path:
            if self.my_cleaner.my_data.add_path(path):
                self.__add_item(path)
                self.text_browser.append(f'{"-" * 115}\n{path}\n{self.set_text.TXT_ADD_PATH}')
            else:
                self.text_browser.append(f'{"-" * 115}\n{self.set_text.TXT_ERR_ADD_PATH}')

    def __add_dir(self) -> None:
        path = QFileDialog.getExistingDirectory(self, self.set_text.TXT_SELECT_DIR)
        self.__add_path(path)

    def __add_files(self) -> None:
        path_tuple = QFileDialog.getOpenFileNames(self, self.set_text.TXT_SELECT_FILES)
        for path in path_tuple[0]:
            self.__add_path(path)

    def _clear_console(self):
        n = '0'
        self.text_browser.setText(self.set_text.TXT_WELCOME)
        self.lcd_dirs.display(n)
        self.lcd_files.display(n)
        self.lcd_errors.display(n)
        self.my_logs.clear()
        self.btn_save_log.setDisabled(True)

    def __add_item(self, item: str) -> None:
        self.list_widget.addItem(item)

    def __remove_items(self) -> None:
        for SelectedItem in self.list_widget.selectedItems():
            self.list_widget.takeItem(self.list_widget.row(SelectedItem))
            self.my_cleaner.my_data.del_data(SelectedItem.text())
            self.text_browser.append(f'{"-" * 115}\n{SelectedItem.text()}\n{self.set_text.TXT_REMOVE_PATH}')

    def __update_lcd(self) -> None:
        self.lcd_dirs.display(str(self.my_cleaner.cleaner.count_del_dirs))
        self.lcd_files.display(str(self.my_cleaner.cleaner.count_del_files))
        self.lcd_errors.display(str(self.my_cleaner.num_errors))

    def __show_msg(self, title: str = 'Warning!', msg: str = 'Message...') -> None:
        QMessageBox.about(self, title, msg)

    def closeEvent(self, event) -> None:
        reply = QMessageBox.question(self, self.set_text.TXT_EXIT, self.set_text.TXT_TO_EXIT,
                                     QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
        if reply == QMessageBox.Yes:
            self.hide()
            self.my_cleaner.wait(1000)
            event.accept()
        else:
            event.ignore()


def main():
    app = QApplication(sys.argv)
    form = MyWindow()
    form.resize(800, 480)
    form.show()
    app.exec_()


if __name__ == '__main__':
    main()
